@@ -15,12 +15,15 @@
 #' predicted) or \emph{NA} (couldn't find stable states in either the drug
 #' combination inhibited model or in any of the two single-drug inhibited models)
 #'
+#' @importFrom tidyr %>%
+#' @importFrom dplyr select
+#' @importFrom tidyselect all_of
 #' @export
 get_observed_model_predictions = function(model.predictions, observed.synergies) {
   drug.combinations.tested = colnames(model.predictions)
-  return(model.predictions[,sapply(drug.combinations.tested, function(drug.comb) {
-    is_comb_element_of(drug.comb, observed.synergies)
-  })])
+  cols = names(which(sapply(drug.combinations.tested,
+    function(drug.comb) is_comb_element_of(drug.comb, observed.synergies))))
+  model.predictions %>% select(all_of(cols))
 }
 
 #' Subset the model predictions to the (false) non-observed synergies
@@ -40,12 +43,15 @@ get_observed_model_predictions = function(model.predictions, observed.synergies)
 #' predicted) or \emph{NA} (couldn't find stable states in either the drug
 #' combination inhibited model or in any of the two single-drug inhibited models)
 #'
+#' @importFrom tidyr %>%
+#' @importFrom dplyr select
+#' @importFrom tidyselect all_of
 #' @export
 get_unobserved_model_predictions = function(model.predictions, observed.synergies) {
   drug.combinations.tested = colnames(model.predictions)
-  return(model.predictions[,sapply(drug.combinations.tested, function(drug.comb) {
-    !is_comb_element_of(drug.comb, observed.synergies)
-  })])
+  cols = names(which(sapply(drug.combinations.tested,
+    function(drug.comb) !is_comb_element_of(drug.comb, observed.synergies))))
+  model.predictions %>% select(all_of(cols))
 }
 
 #' Find the number of predictive models for every synergy subset
@@ -159,7 +165,8 @@ count_models_that_predict_synergies =
 #' \code{observed.model.predictions} and the \code{unobserved.model.predictions}.
 #'
 #' @return a numeric vector of MCC values, each value being in the [-1,1]
-#' interval or \emph{NaN}. The \emph{names} attribute holds the models' names.
+#' interval. The \emph{names} attribute holds the models' names
+#' if applicable (i.e. the input \code{data.frames} have \emph{rownames}).
 #'
 #' @family confusion matrix calculation functions
 #'
@@ -193,8 +200,7 @@ calculate_models_mcc = function(observed.model.predictions, unobserved.model.pre
 
     # Calculate Matthews Correlation Coefficient (MCC)
     models.mcc = calculate_mcc(models.synergies.tp, models.synergies.tn,
-                               models.synergies.fp, models.synergies.fn,
-                               positives, negatives)
+                               models.synergies.fp, models.synergies.fn)
     return(models.mcc)
 }
 
@@ -301,31 +307,32 @@ calculate_models_synergies_tn = function(unobserved.model.predictions) {
   }))
 }
 
-#' Calculate the Matthews correlation coefficient vector
+#' Calculate Matthews correlation coefficient vector
 #'
-#' Use this function to calculate the MCC values given vectors of \emph{TP} (true
-#' positives), \emph{FP} (false positives), \emph{TN} (true negatives), \emph{FN}
-#' (false negatives), \emph{P} (positives) and \emph{N} (negatives). Note that
-#' the input vectors have to be of the same size and have one-to-one value
-#' correspondence for the output MCC vector values to make sense.
+#' Use this function to calculate the MCC scores given vectors of \emph{TP} (true
+#' positives), \emph{FP} (false positives), \emph{TN} (true negatives) and \emph{FN}
+#' (false negatives) values.
+#' Note that the input vectors have to be of the same size and have one-to-one value
+#' correspondence for the output MCC vector to make sense.
 #'
 #' @param tp numeric vector of TPs
 #' @param tn numeric vector of TNs
 #' @param fp numeric vector of FPs
 #' @param fn numeric vector of FNs
-#' @param p numeric vector of positives (p = tp + fn)
-#' @param n numeric vector of negatives (n = tn + fp)
 #'
 #' @return a numeric vector of MCC values, each value being in the [-1,1]
-#' interval or \emph{NaN}.
+#' interval. If any of the four sums of the MCC formula are zero, then we return
+#' an MCC score of zero, which can be shown to be the correct limiting value (model
+#' is no better than a random predictor, see
+#' \href{https://doi.org/10.1186/s12864-019-6413-7}{Chicco et al. (2020)}).
 #'
 #' @family confusion matrix calculation functions
 #'
 #' @export
-calculate_mcc = function(tp, tn, fp, fn, p, n) {
-  return(
-    (tp * tn - fp * fn) / sqrt((tp + fp) * p * n * (tn + fn))
-  )
+calculate_mcc = function(tp, tn, fp, fn) {
+  res = (tp * tn - fp * fn) / sqrt((tp + fp) * (tp + fn) * (tn + fp) * (tn + fn))
+  res[which(!is.finite(res))] = 0 # deal with zero division cases
+  res
 }
 
 #' Update biomarker files for a specific synergy
@@ -340,7 +347,8 @@ calculate_mcc = function(tp, tn, fp, fn, p, n) {
 #' the 'new' biomarkers are added to the 'old' ones.
 #'
 #' @param biomarkers.dir string. It specifies the full path name of the
-#' directory which holds the biomarker files for the synergistic drug combination
+#' directory (without the ending character \emph{/}) which holds the biomarker
+#' files for the synergistic drug combination
 #' specified in the parameter \code{drug.comb}. The biomarker files must be
 #' formatted as: \emph{\%drug.comb\%_biomarkers_active} or
 #' \emph{\%drug.comb\%_biomarkers_inhibited}, where \%drug.comb\% is the value
@@ -377,7 +385,7 @@ update_biomarker_files =
 
     # update the active biomarkers
     active.biomarkers.file =
-      paste0(biomarkers.dir, drug.comb, "_biomarkers_active")
+      paste0(biomarkers.dir, "/", drug.comb, "_biomarkers_active")
 
     if (file.size(active.biomarkers.file) == 0
         || !file.exists(active.biomarkers.file)) {
@@ -428,9 +436,10 @@ update_biomarker_files =
 
     # update the inhibited biomarkers
     inhibited.biomarkers.file =
-      paste0(biomarkers.dir, drug.comb, "_biomarkers_inhibited")
+      paste0(biomarkers.dir, "/", drug.comb, "_biomarkers_inhibited")
 
-    if (file.size(inhibited.biomarkers.file) == 0) {
+    if (file.size(inhibited.biomarkers.file) == 0
+      || !file.exists(inhibited.biomarkers.file)) {
       save_vector_to_file(vector = biomarkers.inhibited.new,
                           file = inhibited.biomarkers.file,
                           with.row.names = TRUE)
